@@ -104,12 +104,12 @@ Your Open edX platform is ready and can be accessed at the following urls:
     "--from",
     "from_version",
     default="koa",
-    type=click.Choice(["ironwood", "juniper", "koa"]),
+    type=click.Choice(["ironwood", "juniper", "koa", "lilac"]),
 )
 @click.option("-I", "--non-interactive", is_flag=True, help="Run non-interactively")
 @click.pass_context
 def upgrade(context: click.Context, from_version: str, non_interactive: bool) -> None:
-    config = tutor_config.load_no_check(context.obj.root)
+    config = tutor_config.load_full(context.obj.root)
 
     if not non_interactive:
         question = """You are about to upgrade your Open edX platform. It is strongly recommended to make a backup before upgrading. To do so, run:
@@ -137,6 +137,21 @@ Are you sure you want to continue?"""
         upgrade_from_koa(context, config)
         running_version = "lilac"
 
+    if running_version == "lilac":
+        # Nothing to do here
+        running_version = "maple"
+
+    if not non_interactive:
+        question = f"""
+Your platform was successfuly upgraded from {from_version} to {running_version}. Depending on your setup, you might have to rebuild some of your Docker images. You can do this now by running the following command in a different shell:
+
+    tutor images build openedx # add your custom images here
+
+Press enter when you are ready to continue"""
+        click.confirm(
+            fmt.question(question), default=True, abort=True, prompt_suffix=" "
+        )
+
 
 def upgrade_from_ironwood(context: click.Context, config: Config) -> None:
     click.echo(fmt.title("Upgrading from Ironwood"))
